use std::path::PathBuf;

use clap::{Args, Parser, Subcommand};

#[derive(Parser)]
pub struct Cli {
    #[arg(
        long = "templates",
        default_value = "./templates",
        value_name = "TEMPLATES_DIR",
        env = "TEMPLATES_DIR"
    )]
    pub templates_dir: PathBuf,
    #[arg(
        long = "static",
        default_value = "./static",
        value_name = "STATIC_DIR",
        env = "STATIC_DIR"
    )]
    pub static_dir: PathBuf,
    #[arg(
        long = "data",
        default_value = "./data",
        value_name = "DATA_DIR",
        env = "DATA_DIR"
    )]
    pub data_dir: PathBuf,

    #[command(subcommand)]
    pub cmd: Subcommands,
}

#[derive(Subcommand)]
pub enum Subcommands {
    Serve { domain: String, port: u16 },
    User(UserCmd),
}

#[derive(Args)]
pub struct UserCmd {
    #[command(subcommand)]
    pub cmd: UserSubCmd,
}

#[derive(Subcommand)]
pub enum UserSubCmd {
    List,
    Remove {
        username: String,
    },
    Add {
        username: String,
        password: String,
        #[arg(long, default_value_t = false)]
        admin: bool,
    },
}
