use chrono::NaiveDate;
use diesel::prelude::*;
use serde::{Deserialize, Serialize};

use crate::db::{schema::*, DbPool, DbResult};

#[derive(Serialize, Queryable, Selectable)]
#[diesel(table_name = images)]
#[diesel(check_for_backend(diesel::sqlite::Sqlite))]
pub struct Image {
    pub id: i32,
    pub plant_id: i32,
    pub date_taken: NaiveDate,
    pub mime_type: String,
    pub note: Option<String>,
}

#[derive(Deserialize, Insertable)]
#[diesel(table_name = images)]
#[diesel(check_for_backend(diesel::sqlite::Sqlite))]
pub struct NewImage {
    plant_id: i32,
    date_taken: NaiveDate,
    mime_type: String,
    note: Option<String>,
}

impl NewImage {
    pub fn new(
        plant_id: i32,
        date_taken: NaiveDate,
        mime_type: String,
        note: Option<String>,
    ) -> Self {
        Self {
            plant_id,
            date_taken,
            mime_type,
            note,
        }
    }

    pub fn insert(self, pool: &DbPool) -> DbResult<Image> {
        Ok(self
            .insert_into(images::table)
            .returning(Image::as_returning())
            .get_result(&mut pool.get()?)?)
    }
}

impl Image {
    pub fn by_id(pool: &DbPool, id: i32) -> DbResult<Option<Self>> {
        Ok(images::table
            .find(id)
            .select(Self::as_select())
            .first(&mut pool.get()?)
            .optional()?)
    }
}
