use diesel::prelude::*;
use serde::{Deserialize, Serialize};

use crate::db::{schema::*, DbPool, DbResult};

#[derive(Serialize, Queryable, Selectable)]
#[diesel(table_name = plants)]
#[diesel(check_for_backend(diesel::sqlite::Sqlite))]
pub struct Plant {
    pub id: i32,
    pub name: String,
    pub species: String,
    pub description: String,
}

#[derive(Deserialize, Insertable)]
#[diesel(table_name = plants)]
#[diesel(check_for_backend(diesel::sqlite::Sqlite))]
pub struct NewPlant {
    name: String,
    species: String,
    description: String,
}

impl NewPlant {
    pub fn insert(self, pool: &DbPool) -> DbResult<Plant> {
        Ok(self
            .insert_into(plants::table)
            .returning(Plant::as_returning())
            .get_result(&mut pool.get()?)?)
    }
}

impl Plant {
    pub fn all(pool: &DbPool) -> DbResult<Vec<Self>> {
        Ok(plants::dsl::plants
            .select(Self::as_select())
            .load(&mut pool.get()?)?)
    }

    pub fn by_id(pool: &DbPool, id: i32) -> DbResult<Option<Self>> {
        Ok(plants::table
            .find(id)
            .select(Self::as_select())
            .first(&mut pool.get()?)
            .optional()?)
    }
}
