use diesel::prelude::*;
use rand::Rng;

use super::user::User;
use crate::db::{schema::*, DbPool, DbResult};

#[derive(Clone, Queryable, Selectable, Insertable, Associations)]
#[diesel(belongs_to(super::user::User))]
#[diesel(table_name = sessions)]
#[diesel(check_for_backend(diesel::sqlite::Sqlite))]
pub struct Session {
    pub id: i64,
    pub user_id: i32,
}

impl Session {
    pub fn new_for_user(pool: &DbPool, user_id: i32) -> DbResult<Self> {
        let id: i64 = rand::thread_rng().gen();

        Ok(Self { id, user_id }
            .insert_into(sessions::table)
            .returning(Self::as_returning())
            .get_result(&mut pool.get()?)?)
    }

    pub fn user_from_id(pool: &DbPool, id: i64) -> DbResult<Option<super::user::User>> {
        Ok(sessions::dsl::sessions
            .inner_join(users::table)
            .filter(sessions::id.eq(id))
            .select(User::as_select())
            .get_result(&mut pool.get()?)
            .optional()?)
    }
}
